#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netconfig.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netconfig.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with netconfig.sh. If not, see <http://www.gnu.org/licenses/>.

echo "Content-Type: application/json"
echo ""

# ./netconfig dhcp|dnsonly|static ip mask [gateway] [dns1] [dns2]
#
# Valid combinations:
# ./netconfing dhcp
#
# ./netconfig dnsonly dns1 [dns2]
#
# ./netconfig static ip mask [gateway] [dns1] [dns2]
#

# source common functions
. ./errlib.sh
. ./netlib.sh
. ./jsonlib.sh

INTERFACE_FILE="/etc/network/interfaces"

config_start() {
    echo "# autogenerated by web setup" > $INTERFACE_FILE
    echo "" >> $INTERFACE_FILE

    if [[ "$?" -ne "0" ]]; then
        return $ERR_FWRITE
    fi

    return $ERR_OK
}

config_loopback()
{
    echo "auto lo" >> $INTERFACE_FILE
    echo "iface lo inet loopback" >> $INTERFACE_FILE
    echo "" >> $INTERFACE_FILE

    if [[ "$?" -ne "0" ]]; then
        return $ERR_FWRITE
    fi

    return $ERR_OK
}

config_dhcp() {
    echo "auto eth0" >> $INTERFACE_FILE
    echo "iface eth0 inet dhcp" >> $INTERFACE_FILE
    echo "" >> $INTERFACE_FILE

    if [[ "$?" -ne "0" ]]; then
        return $ERR_FWRITE
    fi

    return $ERR_OK
}

#1 ip addr, #2 netmask #3 gateway
config_static() {
    echo "auto eth0" >> $INTERFACE_FILE
    echo "iface eth0 inet static" >> $INTERFACE_FILE
    echo "address $1" >> $INTERFACE_FILE
    echo "netmask $2" >> $INTERFACE_FILE
    if [[ -n "$3" ]]; then
        echo "gateway $3" >> $INTERFACE_FILE
    fi
    echo "" >> $INTERFACE_FILE

    if [[ "$?" -ne "0" ]]; then
        return $ERR_FWRITE
    fi

    return $ERR_OK
}

clear_dns() {
    if [[ -e "/etc/resolv.conf" ]]; then
        rm -f /etc/resolv.conf > /dev/null 2>&1
    fi
    echo "# autogenerated by netconfig" > /etc/resolv.conf
    echo "" >> /etc/resolv.conf

    if [[ "$?" -ne "0" ]]; then
        return $ERR_FWRITE
    fi

    return $ERR_OK
}

add_dns() {
    echo "nameserver $1" >> /etc/resolv.conf

    if [[ "$?" -ne "0" ]]; then
        return $ERR_FWRITE
    fi

    return $ERR_OK
}

print_restart_err() {
    if [[ "$1" -eq "$ERR_NETDOWN" ]]; then
         json_error "$1" "Error when shutting down network, but restart succeeded!"
    elif [[ "$1" -eq "$ERR_NETUP" ]]; then
         json_error "$1" "Error bringing up network!"
    else
         json_error "$1" "Unspecified error when restarting network!"
    fi
}

json_ok() {                                         
    echo "{ \"success\": true, \"code\": $ERR_OK, \"message\": \"OK\" }"
}

keep_old_net() {
    OLD_IP=`echo $1 | cut -d ":" -f 3`
    OLD_NETMASK=`echo $1 | cut -d ":" -f 4`
    ifconfig eth0:1 down 1>&2
    ifconfig eth0:1 "$OLD_IP" netmask "$OLD_NETMASK" 1>&2
}

backup_settings() {
    DIFFER=1
    if [[ -f "$INTERFACE_FILE.bkp" ]]; then
        cmp -s "$INTERFACE_FILE" "$INTERFACE_FILE.bkp"
        DIFFER="$?"
    fi

    if [[ "$DIFFER" -eq "1" ]]; then
        cp -f "$INTERFACE_FILE" "$INTERFACE_FILE.bkp" 1>&2
        if [[ "$?" -eq "0" ]]; then
            return $ERR_OK
        else
            return $ERR_BACKUP
        fi
    fi

    return $ERR_OK
}

restore_settings() {
    if [[ -f "$INTERFACE_FILE.bkp" ]]; then
        cp -f "$INTERFACE_FILE.bkp" "$INTERFACE_FILE" 1>&2
        if [[ "$?" -eq "0" ]]; then
            return $ERR_OK
        fi
    fi
        
    return $ERR_RESTORE
}

if [[ "$1" == "dhcp" ]]; then
    config_start
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    config_loopback
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    config_dhcp
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    RET=$(restart_net_with_redirect)
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        if [[ "$RETCODE" -eq "$ERR_NETUP" ]]; then
            keep_old_net $RET
        fi
        if [[ "$RETCODE" -eq "$ERR_NETDOWN" ]]; then
            backup_settings
        fi
        print_restart_err $RETCODE
        exit 1
    fi

    SURVIVE=`echo $RET | cut -d ":" -f 2`
    if [[ "$SURVIVE" -eq "1" ]]; then
        backup_settings
        redirect_and_quit $RET
    fi

    backup_settings
    json_ok

elif [[ "$1" == "dnsonly" ]]; then
    # if both params are empty, the ips will be cleared
    if [[ -z "$2" ]] && [[ -z "$3" ]]; then
        clear_dns
        RETCODE="$?"
        if [[ "$RETCODE" -ne "0" ]]; then
            json_error "$RETCODE" "Could not write configuration!"
            exit 1
        fi

        json_ok
        exit 0
    fi

    # dns1
    validate_ip $2
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_DNS "Invalid dns1 ip address specified!"
        exit 1
    fi
    clear_dns
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    add_dns $2
    RETCODE="$?"
    if [[ "$RETCODE" -ne "0" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    # dns2 (optional)
    if [[ -n "$3" ]]; then
        validate_ip $3
        if [[ "$?" -ne "$ERR_OK" ]]; then
            json_error $ERR_DNS "Invalid dns2 ip address specified!"
            exit 1 
        fi
        add_dns $3
        RETCODE="$?"
        if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
            json_error "$RETCODE" "Could not write configuration!"
            exit 1
        fi
    fi

    json_ok

elif [[ "$1" == "static" ]]; then
    # ip
    validate_ip $2
    if [[ "$?" -ne "$ERR_OK" ]]; then
        json_error $ERR_IP "Invalid ip address specified!"
        exit 1
    fi

    # netmask
    validate_ip $3
    if [[ "$?" -ne "$ERR_OK" ]]; then
        json_error $ERR_MASK "Invalid netmask specified!"
        exit 1
    fi

    # gateway (optional)
    if [[ -n "$4" ]]; then
        validate_ip $4
        if [[ "$?" -ne "$ERR_OK" ]]; then
            json_error $ERR_GW "Invalid gateway ip address specified!"
            exit 1
        fi
    fi

    # dns1 (optional)
    if [[ -n "$5" ]]; then
        validate_ip $5
        if [[ "$?" -ne "$ERR_OK" ]]; then
            json_error $ERR_DNS "Invalid dns1 ip address specified!"
            exit 1
        fi
    fi

    # dns2 (optional)
    if [[ -n "$6" ]]; then
        validate_ip $6
        if [[ "$?" -ne "$ERR_OK" ]]; then
            json_error $ERR_DNS "Invalid dns2 ip address specified!"
            exit 1
        fi
    fi

    config_start
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    config_loopback
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    config_static $2 $3 $4
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not write configuration!"
        exit 1
    fi

    if [[ -n "$5" ]] || [[ -n "$6" ]]; then
        clear_dns
        RETCODE="$?"
        if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
            json_error "$RETCODE" "Could not write configuration!"
            exit 1
        fi
    fi

    if [[ -n "$5" ]]; then
        add_dns $5
        RETCODE="$?"
        if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
            json_error "$RETCODE" "Could not write configuration!"
            exit 1
        fi
    fi
    
    if [[ -n "$6" ]]; then
        add_dns $6
        RETCODE="$?"
        if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
            json_error "$RETCODE" "Could not write configuration!"
            exit 1
        fi
   fi

    RET=$(restart_net_with_redirect)
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        if [[ "$RETCODE" -eq "$ERR_NETUP" ]]; then
            keep_old_net $RET
        fi
        if [[ "$RETCODE" -eq "$ERR_NETDOWN" ]]; then
            backup_settings
        fi
        print_restart_err $RETCODE
        exit 1
    fi

    SURVIVE=`echo $RET | cut -d ":" -f 2`
    if [[ "$SURVIVE" -eq "1" ]]; then
        backup_settings
        redirect_and_quit $RET
    fi

    backup_settings
    json_ok
elif [[ "$1" == "restore" ]]; then
    restore_settings
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not restore configuration!"
        exit 1
    fi

    RET=$(restart_net_with_redirect)
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        if [[ "$RETCODE" -eq "$ERR_NETUP" ]]; then
            keep_old_net $RET
        fi
        print_restart_err $RETCODE
        exit 1
    fi

    SURVIVE=`echo $RET | cut -d ":" -f 2`
    if [[ "$SURVIVE" -eq "1" ]]; then
        redirect_and_quit $RET
    fi

    json_ok
elif [[ "$1" == "backup" ]]; then
    # backup can only be done when we are not in fallback mode
    ALT_IP=`ifconfig eth0:1 | grep 'inet addr:' | cut -d: -f2 | cut -d " " -f1`
    if [[ -n "$ALT_IP" ]]; then
        json_error $ERR_BACKUP "Backup not possible in fallback mode!"
        exit 1
    fi
    
    backup_settings
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error "$RETCODE" "Could not restore configuration!"
        exit 1
    fi

    json_ok
else
    json_error $ERR_PARAM "Invalid option given!"
fi

